Package: leafgl
Title: High-Performance 'WebGl' Rendering for Package 'leaflet'
Version: 0.2.4
Authors@R: 
    c(person(given = "Tim",
             family = "Appelhans",
             role = c("cre", "aut", "cph"),
             email = "tappelhans@tutamail.com",
             comment = c(ORCID = "0000-0002-9824-2707")),
      person(given = "Colin",
             family = "Fay",
             role = "ctb",
             email = "colin@thinkr.fr",
             comment = c(ORCID = "0000-0001-7343-1846")),
      person(given = "Robert",
             family = "Plummer",
             role = "ctb",
             comment = "Leaflet.glify plugin"),
      person(given = "Kent",
             family = "Johnson",
             role = "ctb",
             email = "kjohnson@akoyabio.com"),
      person(given = "Sebastian",
             family = "Gatscha",
             role = "ctb",
             email = "sebastian_gatscha@gmx.at"),
      person(given = "Olivier", 
             family = "Roy", 
             role = "ctb"))
Description: Provides bindings to the 'Leaflet.glify' JavaScript library which extends the 'leaflet' JavaScript library to render large data in the browser using 'WebGl'.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: htmltools, leaflet, sf, yyjsonr, grDevices
Suggests: colourvalues, sp, shiny, testthat (>= 2.1.0)
URL: https://github.com/r-spatial/leafgl,
        https://r-spatial.github.io/leafgl/
BugReports: https://github.com/r-spatial/leafgl/issues
NeedsCompilation: no
Packaged: 2026-04-15 08:22:08 UTC; tim
Author: Tim Appelhans [cre, aut, cph] (ORCID:
    <https://orcid.org/0000-0002-9824-2707>),
  Colin Fay [ctb] (ORCID: <https://orcid.org/0000-0001-7343-1846>),
  Robert Plummer [ctb] (Leaflet.glify plugin),
  Kent Johnson [ctb],
  Sebastian Gatscha [ctb],
  Olivier Roy [ctb]
Maintainer: Tim Appelhans <tappelhans@tutamail.com>
Repository: CRAN
Date/Publication: 2026-04-15 09:10:02 UTC
Built: R 4.6.0; ; 2026-04-28 23:34:38 UTC; unix
