# Generated by roxygen2: do not edit by hand

S3method(makeColorMatrix,Date)
S3method(makeColorMatrix,POSIXct)
S3method(makeColorMatrix,POSIXlt)
S3method(makeColorMatrix,character)
S3method(makeColorMatrix,data.frame)
S3method(makeColorMatrix,factor)
S3method(makeColorMatrix,formula)
S3method(makeColorMatrix,integer)
S3method(makeColorMatrix,json)
S3method(makeColorMatrix,list)
S3method(makeColorMatrix,matrix)
S3method(makeColorMatrix,numeric)
S3method(makePopup,Spatial)
S3method(makePopup,character)
S3method(makePopup,data.frame)
S3method(makePopup,default)
S3method(makePopup,formula)
S3method(makePopup,json)
S3method(makePopup,list)
S3method(makePopup,logical)
S3method(makePopup,matrix)
S3method(makePopup,sf)
S3method(makePopup,shiny.tag)
export(addGlPoints)
export(addGlPolygons)
export(addGlPolylines)
export(clearGlGroup)
export(clearGlLayers)
export(leafglOutput)
export(makeColorMatrix)
export(makePopup)
export(removeGlPoints)
export(removeGlPolygons)
export(removeGlPolylines)
export(renderLeafgl)
importFrom(htmltools,htmlDependencies)
importFrom(htmltools,tagList)
importFrom(htmltools,tags)
importFrom(leaflet,evalFormula)
importFrom(leaflet,leafletOutput)
importFrom(leaflet,renderLeaflet)
