
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom leaflet leafletOutput renderLeaflet evalFormula
#' @importFrom htmltools tagList tags htmlDependencies
## usethis namespace: end
NULL
