% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-popup.R
\name{makePopup}
\alias{makePopup}
\title{makePopup}
\usage{
makePopup(x, data)
}
\arguments{
\item{x}{Object representing the popup}

\item{data}{The dataset}
}
\description{
Transform object to popup
}
